<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://wanotif.id
 * @since      0.1.0
 *
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      0.1.0
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes
 * @author     Wanotif <wanotif.official@gmail.com>
 */
class Wanotif_Woocommerce {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var      Wanotif_Woocommerce_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    0.1.0
	 */
	public function __construct() {
		if ( defined( 'WANOTIF_WOOCOMMERCE_VERSION' ) ) {
			$this->version = WANOTIF_WOOCOMMERCE_VERSION;
		} else {
			$this->version = '1.0.1';
		}
		$this->plugin_name = 'wanotif-woocommerce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wanotif_Woocommerce_Loader. Orchestrates the hooks of the plugin.
	 * - Wanotif_Woocommerce_i18n. Defines internationalization functionality.
	 * - Wanotif_Woocommerce_Admin. Defines all hooks for the admin area.
	 * - Wanotif_Woocommerce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    0.1.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/class-wanotif-woocommerce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/class-wanotif-woocommerce-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'admin/class-wanotif-woocommerce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'public/class-wanotif-woocommerce-public.php';

		/**
		 * This Class responsible to send the massage via wanotif api.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommerce-api.php';

		/**
		 * This class contain a useful function.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommerce-common.php';

		/**
		 * This class responsible for the setting of wantoif api.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'admin/class-wanotif-woocommerce-admin-setting-wa.php';

		/**
		 * This class responsible for the admin of log.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'admin/class-wanotif-woocommerce-admin-log.php';

		/**
		 * This class responsible related order, especialy send a message via WA.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommerce-order.php';

		/**
		 * This class responsible related order, especialy send a message via WA.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommerce-shortcode.php';

		/**
		 * This class responsible for Admin custom shortcode.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'admin/class-wanotif-woocommerce-admin-custom-shortcode.php';

		/**
		 * This file responsible for table api function.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommerce-table-api.php';

		/**
		 * This file responsible custom shortcode table.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommmerce-table-custom-shortcode.php';

		/**
		 * This file responsible log table.
		 */
		require_once WANOTIF_WOOCOMMERCE_PLUGIN_DIR . 'includes/gears/class-wanotif-woocommmerce-table-log.php';

		$this->loader = new Wanotif_Woocommerce_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wanotif_Woocommerce_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    0.1.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wanotif_Woocommerce_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    0.1.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wanotif_Woocommerce_Admin( $this->get_plugin_name(), $this->get_version() );
		$wanotif_woocommerce_admin_setting_wa = new Wanotif_Woocommerce_Admin_Setting_Wa( $this->get_plugin_name(), $this->get_version() );
		$wanotif_woocommerce_order = new Wanotif_Woocommerce_Order( $this->get_plugin_name(), $this->get_version() );
		// $wanotif_woocommerce_admin_custom_shortcode = new Wanotif_Woocommerce_Admin_Custom_Shortcode( $this->get_plugin_name(), $this->get_version() );
		// $wanotif_woocommerce_admin_log = new Wanotif_Woocommerce_Admin_Log( $this->get_plugin_name(), $this->get_version() );
		// $wanotif_wooCommerce_shortcode = new Wanotif_WooCommerce_Shortcode();

		// Show admin menu global agent setting.
		$this->loader->add_action( 'admin_menu', $wanotif_woocommerce_admin_setting_wa, 'add_wa_menu_setting' );
		$this->loader->add_action( 'admin_init', $wanotif_woocommerce_admin_setting_wa, 'register_setup_section' );
		$this->loader->add_action( 'admin_init', $wanotif_woocommerce_admin_setting_wa, 'setup_fields_wa_setting' );

		// add menu on dashboard & register the script.
		$this->loader->add_action( 'admin_menu', $wanotif_woocommerce_admin_custom_shortcode, 'register_the_menu' );
		$this->loader->add_action( 'admin_enqueue_scripts', $wanotif_woocommerce_admin_custom_shortcode, 'register_script_style' );

		$this->loader->add_action( 'admin_menu', $wanotif_woocommerce_admin_log, 'register_the_menu' );
		$this->loader->add_action( 'admin_enqueue_scripts', $wanotif_woocommerce_admin_log, 'register_script_style' );
		$this->loader->add_action( 'wp_ajax_log_request_load_data', $wanotif_woocommerce_admin_log, 'load_data' );

		// Load the Shortcode.
		$this->loader->add_action( 'init', $wanotif_wooCommerce_shortcode, 'register_shortcode' );

		// Ajax for custom shortcode.
		$this->loader->add_action( 'wp_ajax_shrt_load_data', $wanotif_woocommerce_admin_custom_shortcode, 'load_data' );
		$this->loader->add_action( 'wp_ajax_shrt_save_data', $wanotif_woocommerce_admin_custom_shortcode, 'save_data' );
		$this->loader->add_action( 'wp_ajax_shrt_edit_data', $wanotif_woocommerce_admin_custom_shortcode, 'edit_data' );
		$this->loader->add_action( 'wp_ajax_shrt_delete_data', $wanotif_woocommerce_admin_custom_shortcode, 'delete_data' );
		$this->loader->add_action( 'wp_ajax_shrt_load_single_data', $wanotif_woocommerce_admin_custom_shortcode, 'get_single_data' );

		// Register script & style WA page setting.
		$this->loader->add_action( 'admin_enqueue_scripts', $wanotif_woocommerce_admin_setting_wa, 'register_script_style' );

		// Send WA if there any action order.
		$this->loader->add_action( 'woocommerce_order_status_pending', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_failed', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_on-hold', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_completed', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_processing', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_refunded', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );
		$this->loader->add_action( 'woocommerce_order_status_cancelled', $wanotif_woocommerce_order, 'send_order_message_via_wa', 10 );

		// Do send reminder.
		$this->loader->add_action( 'wanotif_reminder_buyer', $wanotif_woocommerce_order, 'remind_the_buyer' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    0.1.0
	 * @access   private
	 */
	private function define_public_hooks() {
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    0.1.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     0.1.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     0.1.0
	 * @return    Wanotif_Woocommerce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     0.1.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
