<?php
/**
 * This file is contain responsible all the action from order related WooCommerce.
 *
 * @link       https://wanotif.id
 * @since      0.2.0
 *
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes/gears/
 */

/**
 * This class responsible for all the action toward order.
 *
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes/gears
 * @author     Wanotif <wanotif.official@gmail.com>
 */
class Wanotif_Woocommerce_Order {

	/**
	 * The ID of this plugin.
	 *
	 * @since    0.2.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    0.2.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize all required perform.
	 *
	 * @since 0.2.0
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Send the order.
	 *
	 * @since 0.2.0
	 * @param integer $order the id post of order.
	 */
	public function send_order_message_via_wa( $order ) {

		$is_wanotif_active = get_option( $this->plugin_name . '_wanotif_active' );

		if ( 'on' !== $is_wanotif_active ) {
			return;
		}

		global $woocommerce;
		$the_text = '';
		$order_id = $order;
		$order = new WC_Order( $order );

		$status = $order->get_status();
		$phone_number = $order->get_billing_phone();

		$wanotif_woocommerce_common = new Wanotif_Woocommerce_Common();
		$status_list = array(
			'pending' => __( 'Pending', 'wanotif-woocommerce' ),
			'failed' => __( 'Failed', 'wanotif-woocommerce' ),
			'on-hold' => 'Receive',
			'processing' => __( 'Processing', 'wanotif-woocommerce' ),
			'completed' => __( 'Completed', 'wanotif-woocommerce' ),
			'refunded' => __( 'Refunded', 'wanotif-woocommerce' ),
			'cancelled' => __( 'Cancelled', 'wanotif-woocommerce' ),
		);

		foreach ( $status_list as $status_lists => $translations ) {
			if ( $status === $status_lists ) {
				$status = $translations;
			}
		}

		switch ( $status ) {
			case __( 'Pending', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-pending' );
				break;

			case __( 'Failed', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-failed' );
				break;

			case 'Receive': // It's on hold.
				$the_text = get_option( $this->plugin_name . '_wa-message-on-hold' );
				break;

			case __( 'Processing', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-processing' );
				break;

			case __( 'Completed', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-completed' );
				break;

			case __( 'Refunded', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-refunded' );
				break;

			case __( 'Cancelled', 'wanotif-woocommerce' ):
				$the_text = get_option( $this->plugin_name . '_wa-message-cancelled' );
				break;

			default:
				break;
		}

		if ( empty( $the_text ) ) {
			return;
		}
		$the_text = do_shortcode( $the_text );
		$shortcode = new Wanotif_WooCommerce_Shortcode();
		$message = $shortcode->process_message_shortcode( $the_text, $order_id );

		// Generate the html to UTF normal encoding.
		$message = $wanotif_woocommerce_common->generate_specific_tag( $message );
		$message = $wanotif_woocommerce_common->set_wa_format_send_api( $message );
		$send = new Wanotif_WooCommerce_Api();
		$send_error_result = $send->send_wa( $phone_number, $message );

		$data_log = array(
			'post_id' => $order_id,
			'status_code' => $send_error_result,
			'message' => $message,
			'phone_number' => $phone_number,
		);

		$this->do_log( $data_log );

	}

	/**
	 * Save the log of wa send.
	 *
	 * @since 0.5.0
	 * @param array $data Contain of the data that needed.
	 */
	public function do_log( $data ) {
		$table = new Wanotif_Woocommerce_Table_Log();
		$status_send_decode = json_decode( $data['status_code'] );

		$status_send = $status_send_decode->wanotif->status;

		$data_log = array(
			'log_post_id' => $data['post_id'],
			'log_phone_number' => $data['phone_number'],
			'log_message' => $data['message'],
		);

		switch ( $status_send ) {
			case 'sent':
				$data_log['log_status'] = true;
				$data_log['log_reason'] = '';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			case 'failed':
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'Chek the phone number';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			case 'Server Not Found.':
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'Check the server name or contact the developer';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			case 'Account Expired.':
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'Your account already expired. renew your account';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			case 'Server not active / ready.':
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'The server still undermaintain';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			case 'API Key Not Valid.':
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'Your apikey is invalide, check on your setting';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
			default:
				$data_log['log_status'] = false;
				$data_log['log_reason'] = 'The error is unknown | try to check if your server can\'t connect us';
				$data_log['log_error_code'] = $status_send;
				$data_log['log_already_send_again'] = false;
				break;
		}

		$table->insert_log( $data_log );

	}

	/**
	 * Check the order and remind the buyer.
	 *
	 * @since 1.0.0
	 */
	public function remind_the_buyer() {

		$is_wanotif_active = get_option( $this->plugin_name . '_wanotif_active' );
		$is_activated = get_option( $plugin_name . '_wanotif_reminder_active' );
		$after_days = (int) get_option( $plugin_name . '_wanotif_reminder_daytosend' );

		if ( 'on' !== $is_activated || 'on' !== $is_wanotif_active || 0 === $after_days || empty( $after_days ) ) {
			return;
		}

		// Do the magic.
		$args = array(
			'post_type'     => 'shop_order',
			'post_status'   => array( 'wc-on-hold' ),
			'no_found_rows' => true,
			'update_post_term_cache' => false,
			'fields' => 'ids',
			'posts_per_page' => 30,
			'meta_query' => array(
				'key' => 'wanotif_remind',
				'compare' => 'NOT EXISTS',
			),
			'date_query'    => array(
				array(
					'column'    => 'post_date_gmt',
					'before'    => $after_days . ' days ago',
				),
			),
		);
		$query = new WP_Query( $args );

		if ( 0 === count( $query->posts ) ) {
			return;
		}
		do_send_reminder( $query->posts );
	}

	/**
	 * Prcess the order to send the reminder message.
	 *
	 * @since 1.0.0
	 * @param array $orders the list of order need to send.
	 */
	public function do_send_reminder( $orders ) {
		global $woocommerce;
		$raw_text = get_option( $this->plugin_name . '_wa-message-reminder' );
		$the_text = do_shortcode( $raw_text );
		$shortcode = new Wanotif_WooCommerce_Shortcode();
		$send = new Wanotif_WooCommerce_Api();
		$wanotif_woocommerce_common = new Wanotif_Woocommerce_Common();

		foreach ( $orders as $order_id ) {
			$order = new WC_Order( $order_id );
			$phone_number = $order->get_billing_phone();

			$message = $shortcode->process_message_shortcode( $the_text, $order_id );

			// Generate the html to UTF normal encoding.
			$message = $wanotif_woocommerce_common->generate_specific_tag( $message );
			$message = $wanotif_woocommerce_common->set_wa_format_send_api( $message );

			$send_error_result = $send->send_wa( $phone_number, $message );

			$data_log = array(
				'post_id' => $order_id,
				'status_code' => $send_error_result,
				'message' => $message,
				'phone_number' => $phone_number,
			);

			$this->do_log( $data_log );
			add_post_meta( $order_id, 'wanotif_remind', '1' );
		}
	}

}
