<?php
/**
 * This class is responsible for all the custom table
 *
 * @link       https://wanotif.id
 * @since      0.4.0
 *
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes/gears/
 */

/**
 * This class is responsible for all the custom table.
 *
 * @since      0.4.0
 * @package    Wanotif_Woocommerce
 * @subpackage Wanotif_Woocommerce/includes/gears
 * @author     Wanotif <wanotif.official@gmail.com>
 */
abstract class Wanotif_Woocommerce_Table_Api {
	/**
	 * Define the name of the table we use.
	 *
	 * @since    0.4.0
	 * @var      string    $table_name  this string define the table name
	 */
	public $table_name;

	/**
	 * Define the name of the primary key id.
	 *
	 * @since    0.4.0
	 * @var      integer    $primary_key  this string define the row name of primary key.
	 */
	public $primary_key;

	/**
	 * Perform All the initial action here.
	 *
	 * @since 0.4.0
	 */
	public function __construct() {}

	/**
	 * Get all the column here.
	 *
	 * @since 0.4.0
	 */
	public function get_columns() {
		return array();
	}

	/**
	 * Get all the default column here.
	 *
	 * @since 0.4.0
	 */
	public function get_column_defaults() {
		return array();
	}

	/**
	 * Get the data id from primary key
	 *
	 * @since 0.4.0
	 * @param integer $row_id the id to search the row.
	 */
	public function get( $row_id ) {
		global $wpdb;
		return $wpdb->get_row(
			$wpdb->prepare(
				'SELECT * FROM %1s WHERE %1s = %2d  LIMIT 1;',
				$this->table_name,
				$this->primary_key,
				$row_id
			)
		);
	}

	/**
	 * Get the data row by column
	 *
	 * @since 0.4.0
	 * @param string $column the column we need to search.
	 * @param mixed  $find_data the value we need to search.
	 */
	public function get_by( $column, $find_data ) {
		global $wpdb;

		return $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM %1s WHERE %2s = '%3s' LIMIT 1;",
				$this->table_name,
				$column,
				$find_data
			)
		);
	}

	/**
	 * Get the column
	 *
	 * @since 0.4.0
	 * @param string  $column the column we need to search.
	 * @param integer $row_id the value we need to search.
	 */
	public function get_column( $column, $row_id ) {
		global $wpdb;
		$column = esc_sql( $column );
		return $wpdb->get_var(
			$wpdb->prepare(
				'SELECT %1s FROM %2s WHERE %3s = %4d LIMIT 1;',
				$column,
				$this->table_name,
				$this->primary_key,
				$row_id
			)
		);
	}

	/**
	 * Get the data with specific column.
	 *
	 * @since 0.4.0
	 * @param string $column the column we need to get.
	 * @param string $column_where column to search.
	 * @param string $column_value the value to search for column.
	 */
	public function get_column_by( $column, $column_where, $column_value ) {
		global $wpdb;
		$column_where = esc_sql( $column_where );
		$column       = esc_sql( $column );
		return $wpdb->get_var(
			$wpdb->prepare(
				"SELECT %1s FROM %2s WHERE %3s = '%4s' LIMIT 1;",
				$column,
				$this->table_name,
				$column_where,
				$column_value
			)
		);
	}

	/**
	 * Insert the data.
	 *
	 * @since 0.4.0
	 * @param array $data The data to insert.
	 */
	public function insert( $data ) {
		global $wpdb;

		// Set Default values.
		$data = wp_parse_args( $data, $this->get_column_defaults() );

		// Initialise column format array.
		$column_formats = $this->get_columns();

		// Force fields to lower case.
		$data = array_change_key_case( $data );

		// White list columns.
		$data = array_intersect_key( $data, $column_formats );

		// Reorder $column_formats to match the ordercolumns given in $data.
		$data_keys = array_keys( $data );
		$column_formats = array_merge( array_flip( $data_keys ), $column_formats );

		$wpdb->insert( $this->table_name, $data, $column_formats );

		return $wpdb->insert_id;
	}

	/**
	 * Update the data.
	 *
	 * @since 0.4.0
	 * @param integer $row_id The target to upadte.
	 * @param array   $data The data to update.
	 * @param string  $where select the target data to update.
	 */
	public function update( $row_id, $data = array(), $where = '' ) {
		global $wpdb;

		// Row ID must be positive integer.
		$row_id = absint( $row_id );

		if ( empty( $row_id ) ) {
			return false;
		}

		if ( empty( $where ) ) {
			$where = $this->primary_key;
		}

		// Initialise column format array.
		$column_formats = $this->get_columns();

		// Force fields to lower case.
		$data = array_change_key_case( $data );

		// White list columns.
		$data = array_intersect_key( $data, $column_formats );

		// Reorder $column_formats to match the order of columns given in $data.
		$data_keys = array_keys( $data );
		$column_formats = array_merge( array_flip( $data_keys ), $column_formats );

		if ( false === $wpdb->update( $this->table_name, $data, array( $where => $row_id ), $column_formats ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Delete the data.
	 *
	 * @since 0.4.0
	 * @param integer $row_id The id target to delete.
	 */
	public function delete( $row_id = 0 ) {
		global $wpdb;

		// Row ID must be positive integer.
		$row_id = absint( $row_id );

		if ( empty( $row_id ) ) {
			return false;
		}

		if ( false === $wpdb->query(
			$wpdb->prepare(
				'DELETE FROM %1s WHERE %2s = %3d ',
				$this->table_name,
				$this->primary_key,
				$row_id
			)
		)
				) {
			return false;
		}

		return true;
	}

	/**
	 * Check if the table exist.
	 *
	 * @since 0.4.0
	 * @param string $table the name of table.
	 */
	public function table_exists( $table ) {
		global $wpdb;
		$table = sanitize_text_field( $table );

		return $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $table ) ) === $table;
	}
}
